@@ -19,10 +19,10 @@ from ._area import (
     meridional_statistics,
     zonal_statistics,
 )
+from ._bias import bias
 from ._cycles import amplitude
 from ._derive import derive
 from ._detrend import detrend
-from ._download import download
 from ._io import (
     _get_debug_filename,
     cleanup,
@@ -42,14 +42,21 @@ from ._mask import (
     mask_multimodel,
     mask_outside_range,
 )
-from ._multimodel import multi_model_statistics
+from ._multimodel import ensemble_statistics, multi_model_statistics
 from ._other import clip
-from ._regrid import extract_levels, extract_point, regrid
+from ._regrid import (
+    extract_coordinate_points,
+    extract_levels,
+    extract_location,
+    extract_point,
+    regrid,
+)
+from ._rolling_window import rolling_window_statistics
 from ._time import (
     annual_statistics,
     anomalies,
     climate_statistics,
-    clip_start_end_year,
+    clip_timerange,
     daily_statistics,
     decadal_statistics,
     extract_month,
@@ -64,8 +71,9 @@ from ._time import (
     timeseries_filter,
 )
 from ._trend import linear_trend, linear_trend_stderr
-from ._units import convert_units
+from ._units import accumulate_coordinate, convert_units
 from ._volume import (
+    axis_statistics,
     depth_integration,
     extract_trajectory,
     extract_transect,
@@ -77,7 +85,6 @@ from ._weighting import weighting_landsea_fraction
 logger = logging.getLogger(__name__)
 
 __all__ = [
-    'download',
     # File reformatting/CMORization
     'fix_file',
     # Load cubes from file
@@ -90,7 +97,7 @@ __all__ = [
     'concatenate',
     'cmor_check_metadata',
     # Extract years given by dataset keys (start_year and end_year)
-    'clip_start_end_year',
+    'clip_timerange',
     # Data reformatting/CMORization
     'fix_data',
     'cmor_check_data',
@@ -115,7 +122,9 @@ __all__ = [
     # Regridding
     'regrid',
     # Point interpolation
+    'extract_coordinate_points',
     'extract_point',
+    'extract_location',
     # Masking missing values
     'mask_multimodel',
     'mask_fillvalues',
@@ -125,6 +134,7 @@ __all__ = [
     'mask_outside_range',
     # Other
     'clip',
+    'rolling_window_statistics',
     # Region selection
     'extract_region',
     'extract_shape',
@@ -134,9 +144,8 @@ __all__ = [
     # 'average_zone': average_zone,
     # 'cross_section': cross_section,
     'detrend',
-    'multi_model_statistics',
-    # Grid-point operations
     'extract_named_regions',
+    'axis_statistics',
     'depth_integration',
     'area_statistics',
     'volume_statistics',
@@ -146,6 +155,7 @@ __all__ = [
     'amplitude',
     'zonal_statistics',
     'meridional_statistics',
+    'accumulate_coordinate',
     'hourly_statistics',
     'daily_statistics',
     'monthly_statistics',
@@ -158,7 +168,14 @@ __all__ = [
     'timeseries_filter',
     'linear_trend',
     'linear_trend_stderr',
+    # Convert units
     'convert_units',
+    # Ensemble statistics
+    'ensemble_statistics',
+    # Multi model statistics
+    'multi_model_statistics',
+    # Bias calculation
+    'bias',
     # Remove fx_variables from cube
     'remove_fx_variables',
     # Save to file
@@ -167,7 +184,7 @@ __all__ = [
 ]
 
 TIME_PREPROCESSORS = [
-    'clip_start_end_year',
+    'clip_timerange',
     'extract_time',
     'extract_season',
     'extract_month',
@@ -182,12 +199,17 @@ TIME_PREPROCESSORS = [
 ]
 
 DEFAULT_ORDER = tuple(__all__)
+"""
+By default, preprocessor functions are applied in this order.
+"""
 
 # The order of initial and final steps cannot be configured
 INITIAL_STEPS = DEFAULT_ORDER[:DEFAULT_ORDER.index('add_fx_variables') + 1]
 FINAL_STEPS = DEFAULT_ORDER[DEFAULT_ORDER.index('remove_fx_variables'):]
 
 MULTI_MODEL_FUNCTIONS = {
+    'bias',
+    'ensemble_statistics',
     'multi_model_statistics',
     'mask_multimodel',
     'mask_fillvalues',
@@ -231,7 +253,8 @@ def check_preprocessor_settings(settings):
                 "function {}".format(missing_args, step))
         # Final sanity check in case the above fails to catch a mistake
         try:
-            inspect.getcallargs(function, None, **settings[step])
+            signature = inspect.Signature.from_callable(function)
+            signature.bind(None, **settings[step])
         except TypeError:
             logger.error(
                 "Wrong preprocessor function arguments in "
@@ -249,7 +272,7 @@ def _check_multi_model_settings(products):
             settings = product.settings.get(step)
             if settings is None:
                 continue
-            elif reference is None:
+            if reference is None:
                 reference = product
             elif reference.settings[step] != settings:
                 raise ValueError(
@@ -273,18 +296,53 @@ def _get_multi_model_settings(products, step):
     return settings, exclude
 
 
-def _run_preproc_function(function, items, kwargs):
+def _run_preproc_function(function, items, kwargs, input_files=None):
     """Run preprocessor function."""
-    msg = "{}({}, {})".format(function.__name__, items, kwargs)
-    logger.debug("Running %s", msg)
+    kwargs_str = ",\n".join(
+        [f"{k} = {pformat(v)}" for (k, v) in kwargs.items()])
+    if input_files is None:
+        file_msg = ""
+    else:
+        file_msg = (f"\nloaded from original input file(s)\n"
+                    f"{pformat(input_files)}")
+    logger.debug(
+        "Running preprocessor function '%s' on the data\n%s%s\nwith function "
+        "argument(s)\n%s", function.__name__, pformat(items), file_msg,
+        kwargs_str)
     try:
         return function(items, **kwargs)
     except Exception:
-        logger.error("Failed to run %s", msg)
+        # To avoid very long error messages, we truncate the arguments and
+        # input files here at a given threshold
+        n_shown_args = 4
+        if input_files is not None and len(input_files) > n_shown_args:
+            n_not_shown_files = len(input_files) - n_shown_args
+            file_msg = (f"\nloaded from original input file(s)\n"
+                        f"{pformat(input_files[:n_shown_args])}\n(and "
+                        f"{n_not_shown_files:d} further file(s) not shown "
+                        f"here; refer to the debug log for a full list)")
+
+        # Make sure that the arguments are indexable
+        if isinstance(items, (PreprocessorFile, Cube, str)):
+            items = [items]
+        if isinstance(items, set):
+            items = list(items)
+
+        if len(items) <= n_shown_args:
+            data_msg = pformat(items)
+        else:
+            n_not_shown_args = len(items) - n_shown_args
+            data_msg = (f"{pformat(items[:n_shown_args])}\n(and "
+                        f"{n_not_shown_args:d} further argument(s) not shown "
+                        f"here; refer to the debug log for a full list)")
+        logger.error(
+            "Failed to run preprocessor function '%s' on the data\n%s%s\nwith "
+            "function argument(s)\n%s", function.__name__, data_msg, file_msg,
+            kwargs_str)
         raise
 
 
-def preprocess(items, step, **settings):
+def preprocess(items, step, input_files=None, **settings):
     """Run preprocessor."""
     logger.debug("Running preprocessor step %s", step)
     function = globals()[step]
@@ -292,10 +350,12 @@ def preprocess(items, step, **settings):
 
     result = []
     if itype.endswith('s'):
-        result.append(_run_preproc_function(function, items, settings))
+        result.append(_run_preproc_function(function, items, settings,
+                                            input_files=input_files))
     else:
         for item in items:
-            result.append(_run_preproc_function(function, item, settings))
+            result.append(_run_preproc_function(function, item, settings,
+                                                input_files=input_files))
 
     items = []
     for item in result:
@@ -324,20 +384,30 @@ def get_step_blocks(steps, order):
 
 class PreprocessorFile(TrackedFile):
     """Preprocessor output file."""
+
     def __init__(self, attributes, settings, ancestors=None):
-        super(PreprocessorFile, self).__init__(attributes['filename'],
-                                               attributes, ancestors)
+        super().__init__(attributes['filename'], attributes, ancestors)
 
         self.settings = copy.deepcopy(settings)
         if 'save' not in self.settings:
             self.settings['save'] = {}
         self.settings['save']['filename'] = self.filename
 
-        self.files = [a.filename for a in ancestors or ()]
+        # self._input_files always contains the original input files;
+        # self.files may change in the preprocessing chain (e.g., by the step
+        # fix_file)
+        self._input_files = [a.filename for a in ancestors or ()]
+        self.files = copy.deepcopy(self._input_files)
 
         self._cubes = None
         self._prepared = False
 
+    def _input_files_for_log(self):
+        """Do not log input files twice in output log."""
+        if self.files == self._input_files:
+            return None
+        return self._input_files
+
     def check(self):
         """Check preprocessor settings."""
         check_preprocessor_settings(self.settings)
@@ -348,7 +418,9 @@ class PreprocessorFile(TrackedFile):
             raise ValueError(
                 "PreprocessorFile {} has no settings for step {}".format(
                     self, step))
-        self.cubes = preprocess(self.cubes, step, **self.settings[step])
+        self.cubes = preprocess(self.cubes, step,
+                                input_files=self._input_files,
+                                **self.settings[step])
         if debug:
             logger.debug("Result %s", self.cubes)
             filename = _get_debug_filename(self.filename, step)
@@ -359,8 +431,10 @@ class PreprocessorFile(TrackedFile):
         if not self._prepared:
             for step in DEFAULT_ORDER[:DEFAULT_ORDER.index('load')]:
                 if step in self.settings:
-                    self.files = preprocess(self.files, step,
-                                            **self.settings[step])
+                    self.files = preprocess(
+                        self.files, step,
+                        input_files=self._input_files_for_log(),
+                        **self.settings[step])
             self._prepared = True
 
     @property
@@ -369,6 +443,7 @@ class PreprocessorFile(TrackedFile):
         if self.is_closed:
             self.prepare()
             self._cubes = preprocess(self.files, 'load',
+                                     input_files=self._input_files_for_log(),
                                      **self.settings.get('load', {}))
         return self._cubes
 
@@ -378,16 +453,19 @@ class PreprocessorFile(TrackedFile):
 
     def save(self):
         """Save cubes to disk."""
-        if self._cubes is not None:
-            self.files = preprocess(self._cubes, 'save',
-                                    **self.settings['save'])
-            self.files = preprocess(self.files, 'cleanup',
-                                    **self.settings.get('cleanup', {}))
+        self.files = preprocess(self._cubes, 'save',
+                                input_files=self._input_files,
+                                **self.settings['save'])
+        self.files = preprocess(self.files, 'cleanup',
+                                input_files=self._input_files,
+                                **self.settings.get('cleanup', {}))
 
     def close(self):
         """Close the file."""
-        self.save()
-        self._cubes = None
+        if self._cubes is not None:
+            self.save()
+            self._cubes = None
+            self.save_provenance()
 
     @property
     def is_closed(self):
@@ -396,13 +474,35 @@ class PreprocessorFile(TrackedFile):
 
     def _initialize_entity(self):
         """Initialize the entity representing the file."""
-        super(PreprocessorFile, self)._initialize_entity()
+        super()._initialize_entity()
         settings = {
             'preprocessor:' + k: str(v)
             for k, v in self.settings.items()
         }
         self.entity.add_attributes(settings)
 
+    def group(self, keys: list) -> str:
+        """Generate group keyword.
+
+        Returns a string that identifies a group. Concatenates a list of
+        values from .attributes
+        """
+        if not keys:
+            return ''
+
+        if isinstance(keys, str):
+            keys = [keys]
+
+        identifier = []
+        for key in keys:
+            attribute = self.attributes.get(key)
+            if attribute:
+                if isinstance(attribute, (list, tuple)):
+                    attribute = '-'.join(attribute)
+                identifier.append(attribute)
+
+        return '_'.join(identifier)
+
 
 # TODO: use a custom ProductSet that raises an exception if you try to
 # add the same Product twice
@@ -429,6 +529,7 @@ def _apply_multimodel(products, step, debug):
 
 class PreprocessingTask(BaseTask):
     """Task for running the preprocessor."""
+
     def __init__(
         self,
         products,
@@ -447,17 +548,44 @@ class PreprocessingTask(BaseTask):
 
     def _initialize_product_provenance(self):
         """Initialize product provenance."""
-        for product in self.products:
-            product.initialize_provenance(self.activity)
+        self._initialize_products(self.products)
+        self._initialize_multimodel_provenance()
+        self._initialize_ensemble_provenance()
 
-        # Hacky way to initialize the multi model products as well.
-        step = 'multi_model_statistics'
-        input_products = [p for p in self.products if step in p.settings]
+    def _initialize_multiproduct_provenance(self, step):
+        input_products = self._get_input_products(step)
         if input_products:
-            statistic_products = input_products[0].settings[step].get(
-                'output_products', {}).values()
-            for product in statistic_products:
-                product.initialize_provenance(self.activity)
+            statistic_products = set()
+
+            for input_product in input_products:
+                step_settings = input_product.settings[step]
+                output_products = step_settings.get('output_products', {})
+
+                for product in output_products.values():
+                    statistic_products.update(product.values())
+
+            self._initialize_products(statistic_products)
+
+    def _initialize_multimodel_provenance(self):
+        """Initialize provenance for multi-model statistics."""
+        step = 'multi_model_statistics'
+        self._initialize_multiproduct_provenance(step)
+
+    def _initialize_ensemble_provenance(self):
+        """Initialize provenance for ensemble statistics."""
+        step = 'ensemble_statistics'
+        self._initialize_multiproduct_provenance(step)
+
+    def _get_input_products(self, step):
+        """Get input products."""
+        return [
+            product for product in self.products if step in product.settings
+        ]
+
+    def _initialize_products(self, products):
+        """Initialize products."""
+        for product in products:
+            product.initialize_provenance(self.activity)
 
     def _run(self, _):
         """Run the preprocessor."""
