@@ -34,15 +34,15 @@ class DerivedVariable(DerivedVariableBase):
     @staticmethod
     def calculate(cubes):
         """Compute Heating from Shortwave Absorption assuming clear sky."""
-        rsdscs_cube = cubes.extract_strict(
+        rsdscs_cube = cubes.extract_cube(
             Constraint(name='surface_downwelling_shortwave_flux_in_air_' +
                        'assuming_clear_sky'))
-        rsdt_cube = cubes.extract_strict(
+        rsdt_cube = cubes.extract_cube(
             Constraint(name='toa_incoming_shortwave_flux'))
-        rsuscs_cube = cubes.extract_strict(
+        rsuscs_cube = cubes.extract_cube(
             Constraint(name='surface_upwelling_shortwave_flux_in_air_' +
                        'assuming_clear_sky'))
-        rsutcs_cube = cubes.extract_strict(
+        rsutcs_cube = cubes.extract_cube(
             Constraint(name='toa_outgoing_shortwave_flux_assuming_clear_sky'))
 
         rsnstcs_cube = (rsdt_cube - rsutcs_cube) - (rsdscs_cube - rsuscs_cube)
