@@ -7,12 +7,13 @@ from itertools import groupby
 from warnings import catch_warnings, filterwarnings
 
 import iris
+import iris.aux_factory
 import iris.exceptions
 import numpy as np
 import yaml
+from cf_units import suppress_errors
 
 from .._task import write_ncl_settings
-from ..cmor._fixes.shared import AtmosphereSigmaFactory
 from ._time import extract_time
 
 logger = logging.getLogger(__name__)
@@ -32,7 +33,6 @@ def _fix_aux_factories(cube):
     """Fix :class:`iris.aux_factory.AuxCoordFactory` after concatenation.
 
     Necessary because of bug in :mod:`iris` (see issue #2478).
-
     """
     coord_names = [coord.name() for coord in cube.coords()]
 
@@ -65,13 +65,14 @@ def _fix_aux_factories(cube):
 
     # Atmosphere sigma coordinate
     if 'atmosphere_sigma_coordinate' in coord_names:
-        new_aux_factory = AtmosphereSigmaFactory(
+        new_aux_factory = iris.aux_factory.AtmosphereSigmaFactory(
             pressure_at_top=cube.coord(var_name='ptop'),
             sigma=cube.coord(var_name='lev'),
             surface_air_pressure=cube.coord(var_name='ps'),
         )
         for aux_factory in cube.aux_factories:
-            if isinstance(aux_factory, AtmosphereSigmaFactory):
+            if isinstance(aux_factory,
+                          iris.aux_factory.AtmosphereSigmaFactory):
                 break
         else:
             cube.add_aux_factory(new_aux_factory)
@@ -89,9 +90,10 @@ def _get_attr_from_field_coord(ncfield, coord_name, attr):
 def concatenate_callback(raw_cube, field, _):
     """Use this callback to fix anything Iris tries to break."""
     # Remove attributes that cause issues with merging and concatenation
-    for attr in ['creation_date', 'tracking_id', 'history']:
-        if attr in raw_cube.attributes:
-            del raw_cube.attributes[attr]
+    _delete_attributes(
+        raw_cube,
+        ('creation_date', 'tracking_id', 'history', 'comment')
+    )
     for coord in raw_cube.coords():
         # Iris chooses to change longitude and latitude units to degrees
         # regardless of value in file, so reinstating file value
@@ -99,28 +101,73 @@ def concatenate_callback(raw_cube, field, _):
             units = _get_attr_from_field_coord(field, coord.var_name, 'units')
             if units is not None:
                 coord.units = units
+        # CMOR sometimes adds a history to the coordinates.
+        _delete_attributes(coord, ('history', ))
 
 
-def load(file, callback=None):
-    """Load iris cubes from files."""
+def _delete_attributes(iris_object, atts):
+    for att in atts:
+        if att in iris_object.attributes:
+            del iris_object.attributes[att]
+
+
+def load(file, callback=None, ignore_warnings=None):
+    """Load iris cubes from files.
+
+    Parameters
+    ----------
+    file: str
+        File to be loaded.
+    callback: callable or None, optional (default: None)
+        Callback function passed to :func:`iris.load_raw`.
+    ignore_warnings: list of dict or None, optional (default: None)
+        Keyword arguments passed to :func:`warnings.filterwarnings` used to
+        ignore warnings issued by :func:`iris.load_raw`. Each list element
+        corresponds to one call to :func:`warnings.filterwarnings`.
+
+    Returns
+    -------
+    iris.cube.CubeList
+        Loaded cubes.
+
+    Raises
+    ------
+    ValueError
+        Cubes are empty.
+    """
     logger.debug("Loading:\n%s", file)
+    if ignore_warnings is None:
+        ignore_warnings = []
+
+    # Avoid duplication of ignored warnings when load() is called more often
+    # than once
+    ignore_warnings = list(ignore_warnings)
+
+    # Default warnings ignored for every dataset
+    ignore_warnings.append({
+        'message': "Missing CF-netCDF measure variable .*",
+        'category': UserWarning,
+        'module': 'iris',
+    })
+    ignore_warnings.append({
+        'message': "Ignoring netCDF variable '.*' invalid units '.*'",
+        'category': UserWarning,
+        'module': 'iris',
+    })
+
+    # Filter warnings
     with catch_warnings():
-        filterwarnings(
-            'ignore',
-            message="Missing CF-netCDF measure variable .*",
-            category=UserWarning,
-            module='iris',
-        )
-        filterwarnings(
-            'ignore',
-            message="Ignoring netCDF variable '.*' invalid units '.*'",
-            category=UserWarning,
-            module='iris',
-        )
-        raw_cubes = iris.load_raw(file, callback=callback)
+        for warning_kwargs in ignore_warnings:
+            warning_kwargs.setdefault('action', 'ignore')
+            filterwarnings(**warning_kwargs)
+        # Suppress UDUNITS-2 error messages that cannot be ignored with
+        # warnings.filterwarnings
+        # (see https://github.com/SciTools/cf-units/issues/240)
+        with suppress_errors():
+            raw_cubes = iris.load_raw(file, callback=callback)
     logger.debug("Done with loading %s", file)
     if not raw_cubes:
-        raise Exception('Can not load cubes from {0}'.format(file))
+        raise ValueError(f'Can not load cubes from {file}')
     for cube in raw_cubes:
         cube.attributes['source_file'] = file
     return raw_cubes
@@ -199,10 +246,13 @@ def concatenate(cubes):
     return result
 
 
-def save(cubes, filename, optimize_access='', compress=False, alias='',
+def save(cubes,
+         filename,
+         optimize_access='',
+         compress=False,
+         alias='',
          **kwargs):
-    """
-    Save iris cubes to file.
+    """Save iris cubes to file.
 
     Parameters
     ----------
@@ -224,6 +274,9 @@ def save(cubes, filename, optimize_access='', compress=False, alias='',
     compress: bool, optional
         Use NetCDF internal compression.
 
+    alias: str, optional
+        Var name to use when saving instead of the one in the cube.
+
     Returns
     -------
     str
@@ -233,7 +286,6 @@ def save(cubes, filename, optimize_access='', compress=False, alias='',
     ------
     ValueError
         cubes is empty.
-
     """
     if not cubes:
         raise ValueError(f"Cannot save empty cubes '{cubes}'")
@@ -276,8 +328,8 @@ def save(cubes, filename, optimize_access='', compress=False, alias='',
     if alias:
 
         for cube in cubes:
-            logger.debug(
-                'Changing var_name from %s to %s', cube.var_name, alias)
+            logger.debug('Changing var_name from %s to %s', cube.var_name,
+                         alias)
             cube.var_name = alias
     iris.save(cubes, **kwargs)
 
@@ -448,6 +500,9 @@ def _concatenate_overlapping_cubes(cubes):
                                     data_start_1.month, data_start_1.day,
                                     start_overlap.year, start_overlap.month,
                                     start_overlap.day)
+            # convert c1_delta scalar cube to vector cube, if needed
+            if c1_delta.data.shape == ():
+                c1_delta = iris.util.new_axis(c1_delta, scalar_coord="time")
             cubes = iris.cube.CubeList([c1_delta, cubes[1]])
             logger.debug("Attempting concatenatenation of %s with %s",
                          c1_delta, cubes[1])
