@@ -71,9 +71,10 @@ class Test(tests.Test):
         self.assert_array_equal(result.data, expected)
 
     def test_area_statistics_cell_measure_mean(self):
+        """Test for area average of a 2D field.
+
+        The area measure is pre-loaded in the cube
         """
-        Test for area average of a 2D field.
-        The area measure is pre-loaded in the cube"""
         cube = guess_bounds(self.grid, ['longitude', 'latitude'])
         grid_areas = iris.analysis.cartography.area_weights(cube)
         measure = iris.coords.CellMeasure(
@@ -143,10 +144,8 @@ class Test(tests.Test):
         self.assert_array_equal(result.data, expected)
 
     def test_extract_region_mean(self):
-        """
-        Test for extracting a region and performing
-        the area mean of a 2D field.
-        """
+        """Test for extracting a region and performing the area mean of a 2D
+        field."""
         cube = guess_bounds(self.grid, ['longitude', 'latitude'])
         grid_areas = iris.analysis.cartography.area_weights(cube)
         measure = iris.coords.CellMeasure(
@@ -709,6 +708,41 @@ def test_extract_shape_natural_earth(make_testcube, ne_ocean_shapefile):
     np.testing.assert_array_equal(result.data.data, expected)
 
 
+def test_extract_shape_fx(make_testcube, ne_ocean_shapefile):
+    """Test for extracting a shape from NE file."""
+    expected = np.ones((5, 5))
+    preproc_path = Path(esmvalcore.preprocessor.__file__).parent
+    shp_file = preproc_path / "ne_masks" / "ne_50m_ocean.shp"
+    cube = make_testcube
+    measure = iris.coords.CellMeasure(cube.data,
+                                      standard_name='cell_area',
+                                      var_name='areacello',
+                                      units='m2',
+                                      measure='area')
+    ancillary_var = iris.coords.AncillaryVariable(
+        cube.data,
+        standard_name='land_ice_area_fraction',
+        var_name='sftgif',
+        units='%')
+    cube.add_cell_measure(measure, (0, 1))
+    cube.add_ancillary_variable(ancillary_var, (0, 1))
+    result = extract_shape(
+        cube,
+        shp_file,
+        crop=False,
+    )
+    np.testing.assert_array_equal(result.data.data, expected)
+
+    assert result.cell_measures()
+    result_measure = result.cell_measure('cell_area').data
+    np.testing.assert_array_equal(measure.data, result_measure)
+
+    assert result.ancillary_variables()
+    result_ancillary_var = result.ancillary_variable(
+        'land_ice_area_fraction').data
+    np.testing.assert_array_equal(ancillary_var.data, result_ancillary_var)
+
+
 def test_extract_shape_ne_check_nans(ne_ocean_shapefile):
     """Test shape from NE file with check for boundary NaN's."""
     cube = _create_sample_full_cube()
@@ -793,30 +827,34 @@ def test_extract_specific_shape(make_testcube, tmp_path, ids):
     polyg = []
     for n in range(nshape):
         polyg.append(
-            Polygon([(1.0 + n, 1.0 + slat), (1.0 + n, 1.0),
-                     (1.0 + n + slon, 1.0), (1.0 + n + slon, 1.0 + slat)]))
+            Polygon([
+                (1.0 + n, 1.0 + slat),
+                (1.0 + n, 1.0),
+                (1.0 + n + slon, 1.0),
+                (1.0 + n + slon, 1.0 + slat),
+            ])
+        )
     write_shapefile(polyg, tmp_path / 'test_shape.shp')
 
-    # Make corresponding expected masked array
-    (slat, slon) = np.ceil([slat, slon]).astype(int)
-    vals = np.ones((nshape, min(slat + 2, 5), min(slon + 1 + nshape, 5)))
-    mask = vals.copy()
-    for n in ids:
-        mask[n, 1:1 + slat, 1 + n:1 + n + slon] = 0
-    expected = np.ma.masked_array(vals, mask)
-
-    # this detour is necessary, otherwise the data will not agree
-    data = expected.data.max(axis=0)
-    mask = expected.max(axis=0).mask
-    expected = np.ma.masked_array(data=data, mask=mask)
-
     result = extract_shape(make_testcube,
                            tmp_path / 'test_shape.shp',
                            crop=True,
                            decomposed=False,
                            ids=ids)
-    np.testing.assert_array_equal(result.data.data, expected.data)
-    np.testing.assert_array_equal(result.data.mask, expected.mask)
+
+    expected_bounds = np.vstack([polyg[i].bounds for i in ids])
+
+    lon_min = expected_bounds[:, 0]
+    lat_min = expected_bounds[:, 1]
+    lon_max = expected_bounds[:, 2]
+    lat_max = expected_bounds[:, 3]
+
+    # results from `extract_shape` are padded with masked values
+    lats = result.coord('latitude')[1:-1]
+    lons = result.coord('longitude')[1:-1]
+
+    assert np.all((lats.points >= lat_min) & (lats.points <= lat_max))
+    assert np.all((lons.points >= lon_min) & (lons.points <= lon_max))
 
 
 def test_extract_specific_shape_raises_if_not_present(make_testcube, tmp_path):
